from django.core.management.base import BaseCommand, CommandError
from django.db import transaction
from django_purge import models


class Command(BaseCommand):
    help = "Run all database purgers"

    def handle(self, *args, **options):
        """Run all database purgers"""
        models.DatabasePurger.purge_all()



# Generated by Django 2.1.3 on 2019-02-07 22:17

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='DatabasePurger',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('enabled', models.BooleanField(default=True)),
                ('delete_by_age', models.BooleanField(default=True, help_text='Delete if entry is older than specified age')),
                ('delete_by_quantity', models.BooleanField(default=False, help_text='Delete all except the youngest `max_records` entries')),
                ('datetime_field', models.CharField(default='created', help_text='Field used to determine the age of a record', max_length=255)),
                ('age_in_days', models.PositiveIntegerField(default=30, help_text='If `delete_by_age` is selected, delete records older than this age')),
                ('max_records', models.PositiveIntegerField(default=1000, help_text='Number of records to keep if `delete_by_quantity` is selected')),
                ('tables', models.ManyToManyField(to='contenttypes.ContentType')),
            ],
            options={
                'verbose_name': 'Database Purger',
            },
        ),
    ]



"""
This module is designed to be used by this app. The purpose of this module is
to provide default values for the settings, and to implement a hook so other
modules in the app need only import this settings module, rather than the
Django project settings as well.

This module also provides settings needed to run this app without the need for
a larger project context.

Generated by 'django-admin startproject' using Django 2.0.4.
"""

try:
    from django.conf import settings
except ImportError:
    class settings: pass
import os
import sys


def get_setting(name):
    """
    Hook for getting Django settings without modules having to import it.
    Project settings dominate, otherwise use the default here.
    """
    me = sys.modules[__name__]
    return getattr(settings, name, getattr(me, name, None))


# setting `base_dir` before defaults since it needs to be used by `DATABASES`.
base_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# This section defines settings required for the app to run standalone.
BASE_DIR = base_dir
SECRET_KEY = 'not-a-very-good-secret'
DEBUG = True
ALLOWED_HOSTS = ['*']
INSTALLED_APPS = [
    'django_purge',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]
MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]
ROOT_URLCONF = 'django_purge.urls'
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]
WSGI_APPLICATION = 'django_purge.wsgi.application'
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(base_dir, 'db.sqlite3'),
    }
}
AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]
LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'America/Chicago'
USE_I18N = True
USE_L10N = True
USE_TZ = True
STATIC_URL = '/static/'
APPEND_SLASH = True

CRON_CLASSES = [
    'django_purge.cron.CronJob',
]



from django.contrib import admin
from django.contrib.admin.widgets import FilteredSelectMultiple
from django.contrib.contenttypes.models import ContentType
from django.forms import ModelMultipleChoiceField
from . import models


class CustomModelMCF(ModelMultipleChoiceField):
    widget = FilteredSelectMultiple("Models", False)

    def label_from_instance(self, obj):
        return "{0} :: {1}".format(obj.app_label, obj)


class DatabasePurgerAdmin(admin.ModelAdmin):
    list_filter = ('enabled',)
    list_display = ('name',) + list_filter + ('selected_tables', 'delete_by_age', 'delete_by_quantity', 'datetime_field', 'age_in_days', 'max_records')
    search_fields = list_display
    fieldsets = (
        (None, { 'fields': ('name', 'enabled', 'tables')}),
        ('Criteria', { 'fields': ('delete_by_age', 'delete_by_quantity', 'datetime_field', 'age_in_days', 'max_records')}),
    )

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == 'tables':
            return CustomModelMCF(ContentType.objects.all(), **kwargs)
        return super().formfield_for_manytomany(db_field, request, **kwargs)


admin.site.register(models.DatabasePurger, DatabasePurgerAdmin)



from django.core.management import call_command
from django_cron import CronJobBase, Schedule
from .models import DatabasePurger


class CronJob(CronJobBase):
    schedule = Schedule(run_every_mins=240)
    code = 'django_purge.cron'

    def do(self):
        call_command('purge')
